// let domain be assigned by browser unless specified
const secureAttributes = ["secure", "__Secure-", "samesite"];
const strictPathAttributes = ["path", "__Host-"];
/**
 * setCookieAttributes
 * @description a cookie attribute manager
 * @param {object} attrs cookie attributes
 * @returns {string}
 */
export function setCookieAttributes(attrs = {}) {
    const items = Object.keys(attrs);
    if (!items.length)
        return "";
    // set secure attributes
    const isSecureProtocol = document.location.protocol === "https:";
    const hasIsSecureAttr = items.find(item => item === "isSecure") && isSecureProtocol;
    const hasIsStrict = items.find(item => item === "isStrictSecure") && isSecureProtocol;
    let updatedAttrs = items;
    if (hasIsStrict) {
        // merge secure attributes
        // remove domain if added
        updatedAttrs = items.concat(secureAttributes, strictPathAttributes).filter(item => item !== 'domain');
    }
    else if (hasIsSecureAttr) {
        updatedAttrs = items.concat(secureAttributes);
    }
    // prefer days over expires attribute
    const filteredAttrs = updatedAttrs.every((attr) => ["days", "expires"].includes(attr))
        ? updatedAttrs.filter((attr) => attr !== "expires")
        : updatedAttrs;
    // return attributes with booleans  and key value pairs
    return filteredAttrs.reduce((str, attr) => {
        const isTruthyAttr = secureAttributes.concat(["__Host-"]).includes(attr);
        // return truthy attributes w/o values
        if (isTruthyAttr)
            str += `; ${attr}`;
        // secure specific values
        if (attr === 'sameSite' && hasIsSecureAttr)
            str += `; ${attr}=strict`;
        if (attr === 'path' && hasIsStrict)
            str += `; ${attr}=/`;
        // return days helper as expires
        else if (attr === "days")
            str += `; expires=${new Date(Date.now() + attrs[attr] || 0 * 864e5).toUTCString()};`;
        // return all other key value string pairs
        else
            str += `; ${attr}=${attrs[attr]}`;
        return str;
    }, "");
}
/**
 * setCookieList
 * @description returns a cookie dictionary
 * @returns {object}
 */
export function setCookieList() {
    return document.cookie
        .split(";")
        .map((cookie) => cookie.split("="))
        .reduce((list, [key, value]) => ({
        ...list,
        [key.trim()]: decodeURIComponent(value),
    }), {});
}
export function setUpdatedState({ id, name, value, attrs }) {
    const currentStorage = localStorage.getItem(id);
    const currentState = (currentStorage ? JSON.parse(currentStorage) : {});
    if (value) {
        const updatedState = {
            ...currentState,
            [name]: {
                name,
                value,
                ...(Object.keys(attrs).length ? { attrs } : {}),
            },
        };
        localStorage.setItem(id, JSON.stringify(updatedState));
    }
    else if (Object.keys(currentState).length) {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const { [name]: deleted, ...updatedState } = currentState;
        localStorage.setItem(id, JSON.stringify(updatedState));
    }
}
